#!/usr/bin/perl
#################################################
#
# GAIM Buddy List Sorter
#
# Written By: John Resig (phytar@csh.rit.edu)
# Date: April 12, 2003
#
# For More Information, please visit the 
# following web page:
#    http://csh.rit.edu/~phytar/
#
# This program can be openly distributed and
# edited, as long as it keeps the original
# author and contact information.
#
#################################################

if ( $#ARGV < 0) {
	&error( "Usage: perl sortGaim.pl [-s SORTTYPE] [-i] [-w] FILE
	-s SORTTYPE = 'ascending' (default) OR 'descending'
	-i = Turns off ignoring username case
	-w = Turns off auto-writing to file" );
}

$ignoreCase = 1;
$autoWrite = 1;

for ( $i = 0; $i <= $#ARGV; $i++ ) {
	if ( $ARGV[ $i ] eq "-s" ) {
		$sortType = $ARGV[ ++$i ];
	} elsif ( $ARGV[ $i ] eq "-i" ) {
		$ignoreCase = 0;
	} elsif ( $ARGV[ $i ] eq "-w" ) {
		$autoWrite = 0;
	} elsif ( $i == $#ARGV ) {
		$inFile = $ARGV[ $i ];
	}
}

open ( FILE, $inFile ) || &error( "Error: $!" );
$file = join('', <FILE> );
close ( FILE );

$file =~ s/<group name="([^"]*)">(.*?)<\/group>/&sortUsers($1,$2)/egs;

if ( $autoWrite ) {
	open ( FILE, ">" . $inFile ) || &error( "Error: $!" );
	print FILE $file;
	close ( FILE );
} else {
	print $file;
}

sub sortUsers {
	my ( $groupInfo, $gXML ) = @_;
	my %users = (), $retVal = "";
	
	while ( $gXML =~ s/<person name="([^"]*)">(.*?)<\/person>//s ) {
		if ( $ignoreCase ) {
			$users{ lc( $1 ) } = $2;
		} else {
			$users{ $1 } = $2;
		}
	}

	$retVal = qq~<group name="$groupInfo">\n~;
	if ( $sortType eq "descending" ) {
		foreach $user ( reverse sort keys %users ) {
			$retVal .= qq~\t\t\t<person name="$user">$users{$user}</person>\n~;
		}
	} else {
		foreach $user ( sort keys %users ) {
			$retVal .= qq~\t\t\t<person name="$user">$users{$user}</person>\n~;
		}
	}
	$retVal .= "\t\t</group>";
	
	return $retVal;
}

sub error {
	print $_[ 0 ] . "\n";
	exit( 0 );
}